@@ -10,6 +10,21 @@ ffibuild.set_source(
 )
 
 ffibuild.cdef("""
+typedef struct ncinput {
+  char32_t id;     // Unicode codepoint
+  int y;           // Y cell coordinate of event, -1 for undefined
+  int x;           // X cell coordinate of event, -1 for undefined
+  bool alt;        // Was Alt held during the event?
+  bool shift;      // Was Shift held during the event?
+  bool ctrl;       // Was Ctrl held during the event?
+  uint64_t seqnum; // Monotonically increasing input event counter
+} ncinput;
+bool nckey_mouse_p(char32_t r);
+// sigset_t differs from system to system, annoying
+// char32_t notcurses_getc(struct notcurses* n, const struct timespec* ts, sigset_t* sigmask, ncinput* ni);
+char32_t notcurses_getc_nblock(struct notcurses* n, ncinput* ni);
+char32_t notcurses_getc_blocking(struct notcurses* n, ncinput* ni);
+int notcurses_inputready_fd(struct notcurses* n);
 typedef struct cell {
   // These 32 bits are either a single-byte, single-character grapheme cluster
   // (values 0--0x7f), or an offset into a per-ncplane attached pool of
@@ -50,22 +65,6 @@ typedef struct notcurses_options {
   // the environment variable TERM is used. Failure to open the terminal
   // definition will result in failure to initialize notcurses.
   const char* termtype;
-  // If smcup/rmcup capabilities are indicated, notcurses defaults to making
-  // use of the "alternate screen". This flag inhibits use of smcup/rmcup.
-  bool inhibit_alternate_screen;
-  // By default, we hide the cursor if possible. This flag inhibits use of
-  // the civis capability, retaining the cursor.
-  bool retain_cursor;
-  // Notcurses typically prints version info in notcurses_init() and performance
-  // info in notcurses_stop(). This inhibits that output.
-  bool suppress_banner;
-  // We typically install a signal handler for SIG{INT, SEGV, ABRT, QUIT} that
-  // restores the screen, and then calls the old signal handler. Set to inhibit
-  // registration of these signal handlers.
-  bool no_quit_sighandlers;
-  // We typically install a signal handler for SIGWINCH that generates a resize
-  // event in the notcurses_getc() queue. Set to inhibit this handler.
-  bool no_winch_sighandler;
   // If non-NULL, notcurses_render() will write each rendered frame to this
   // FILE* in addition to outfp. This is used primarily for debugging.
   FILE* renderfp;
@@ -77,21 +76,17 @@ typedef struct notcurses_options {
   // strictly best-effort. Absolute coordinates are relative to the rendering
   // area ((0, 0) is always the origin of the rendering area).
   int margin_t, margin_r, margin_b, margin_l;
+  // General flags; see NCOPTION_*. This is expressed as a bitfield so that
+  // future options can be added without reshaping the struct. Undefined bits
+  // must be set to 0.
+  uint64_t flags;
 } notcurses_options;
 struct notcurses* notcurses_init(const notcurses_options*, FILE*);
+int notcurses_lex_margins(const char* op, notcurses_options* opts);
 int notcurses_stop(struct notcurses*);
 int notcurses_render(struct notcurses*);
 struct ncplane* notcurses_stdplane(struct notcurses*);
 const struct ncplane* notcurses_stdplane_const(const struct notcurses* nc);
-typedef struct ncinput {
-  char32_t id;     // identifier. Unicode codepoint or synthesized NCKEY event
-  int y;           // y cell coordinate of event, -1 for undefined
-  int x;           // x cell coordinate of event, -1 for undefined
-  bool shift;
-  bool alt;
-  bool ctrl;
-  uint64_t seqnum;
-} ncinput;
 int ncplane_set_base_cell(struct ncplane* ncp, const cell* c);
 int ncplane_set_base(struct ncplane* ncp, const char* egc, uint32_t attrword, uint64_t channels);
 int ncplane_base(struct ncplane* ncp, cell* c);
@@ -109,11 +104,16 @@ typedef enum {
 struct ncplane* ncplane_aligned(struct ncplane* n, int rows, int cols, int yoff, ncalign_e align, void* opaque);
 unsigned notcurses_supported_styles(const struct notcurses* nc);
 int notcurses_palette_size(const struct notcurses* nc);
+bool notcurses_cantruecolor(const struct notcurses* nc);
 bool notcurses_canfade(const struct notcurses* nc);
+bool notcurses_canchangecolor(const struct notcurses* nc);
+bool notcurses_canopen_images(const struct notcurses* nc);
+bool notcurses_canopen_videos(const struct notcurses* nc);
+bool notcurses_canutf8(const struct notcurses* nc);
+bool notcurses_cansixel(const struct notcurses* nc);
 int notcurses_mouse_enable(struct notcurses* n);
 int notcurses_mouse_disable(struct notcurses* n);
 int ncplane_destroy(struct ncplane* ncp);
-bool notcurses_canopen(const struct notcurses* nc);
 int ncplane_mergedown(struct ncplane* restrict src, struct ncplane* restrict dst);
 void ncplane_erase(struct ncplane* n);
 int ncplane_cursor_move_yx(struct ncplane* n, int y, int x);
@@ -122,16 +122,28 @@ int ncplane_move_yx(struct ncplane* n, int y, int x);
 void ncplane_yx(struct ncplane* n, int* y, int* x);
 void ncplane_dim_yx(const struct ncplane* n, int* rows, int* cols);
 int ncplane_putc_yx(struct ncplane* n, int y, int x, const cell* c);
-int ncplane_move_top(struct ncplane* n);
-int ncplane_move_bottom(struct ncplane* n);
-int ncplane_move_below(struct ncplane* n, struct ncplane* below);
-int ncplane_move_above(struct ncplane* n, struct ncplane* above);
+void ncplane_move_top(struct ncplane* n);
+void ncplane_move_bottom(struct ncplane* n);
+int ncplane_move_below(struct ncplane* restrict n, struct ncplane* restrict below);
+int ncplane_move_above(struct ncplane* restrict n, struct ncplane* restrict above);
 struct ncplane* ncplane_below(struct ncplane* n);
 char* notcurses_at_yx(struct notcurses* nc, int yoff, int xoff, uint32_t* attrword, uint64_t* channels);
 char* ncplane_at_cursor(struct ncplane* n, uint32_t* attrword, uint64_t* channels);
 int ncplane_at_cursor_cell(struct ncplane* n, cell* c);
-char* ncplane_at_yx(struct ncplane* n, int y, int x, uint32_t* attrword, uint64_t* channels);
+char* ncplane_at_yx(const struct ncplane* n, int y, int x, uint32_t* attrword, uint64_t* channels);
 int ncplane_at_yx_cell(struct ncplane* n, int y, int x, cell* c);
+typedef enum {
+  NCBLIT_1x1,     // full block                █
+  NCBLIT_2x1,     // full/(upper|left) blocks  ▄█
+  NCBLIT_1x1x4,   // shaded full blocks        ▓▒░█
+  NCBLIT_2x2,     // quadrants                 ▗▐ ▖▄▟▌▙█
+  NCBLIT_4x1,     // four vert/horz levels     █▆▄▂ / ▎▌▊█
+  NCBLIT_BRAILLE, // 4 rows, 2 cols (braille)  ⡀⡄⡆⡇⢀⣀⣄⣆⣇⢠⣠⣤⣦⣧⢰⣰⣴⣶⣷⢸⣸⣼⣾⣿
+  NCBLIT_8x1,     // eight vert/horz levels    █▇▆▅▄▃▂▁ / ▏▎▍▌▋▊▉█
+  NCBLIT_SIXEL,   // 6 rows, 1 col (RGB)
+} ncblitter_e;
+uint32_t* ncplane_rgba(const struct ncplane* nc, ncblitter_e blit, int begy, int begx, int leny, int lenx);
+char* ncplane_contents(const struct ncplane* nc, int begy, int begx, int leny, int lenx);
 void* ncplane_set_userptr(struct ncplane* n, void* opaque);
 void* ncplane_userptr(struct ncplane* n);
 int ncplane_resize(struct ncplane* n, int keepy, int keepx, int keepleny,
@@ -154,8 +166,8 @@ int ncplane_set_fg(struct ncplane* n, unsigned channel);
 int ncplane_set_bg(struct ncplane* n, unsigned channel);
 void ncplane_set_fg_default(struct ncplane* n);
 void ncplane_set_bg_default(struct ncplane* n);
-int ncplane_set_fg_alpha(struct ncplane* n, int alpha);
-int ncplane_set_bg_alpha(struct ncplane* n, int alpha);
+int ncplane_set_fg_alpha(struct ncplane* n, unsigned alpha);
+int ncplane_set_bg_alpha(struct ncplane* n, unsigned alpha);
 int ncplane_set_fg_palindex(struct ncplane* n, int idx);
 int ncplane_set_bg_palindex(struct ncplane* n, int idx);
 void ncplane_styles_set(struct ncplane* n, unsigned stylebits);
@@ -186,7 +198,7 @@ void notcurses_reset_stats(struct notcurses* nc, ncstats* stats);
 int ncplane_hline_interp(struct ncplane* n, const cell* c, int len, uint64_t c1, uint64_t c2);
 int ncplane_vline_interp(struct ncplane* n, const cell* c, int len, uint64_t c1, uint64_t c2);
 int ncplane_box(struct ncplane* n, const cell* ul, const cell* ur, const cell* ll, const cell* lr, const cell* hline, const cell* vline, int ystop, int xstop, unsigned ctlword);
-typedef int (*fadecb)(struct notcurses* nc, struct ncplane* ncp, void* curry);
+typedef int (*fadecb)(struct notcurses* nc, struct ncplane* ncp, const struct timespec* ts, void* curry);
 int ncplane_fadeout(struct ncplane* n, const struct timespec* ts, fadecb fader, void* curry);
 int ncplane_fadein(struct ncplane* n, const struct timespec* ts, fadecb fader, void* curry);
 int ncplane_pulse(struct ncplane* n, const struct timespec* ts, fadecb fader, void* curry);
@@ -194,6 +206,7 @@ int ncplane_putwc_yx(struct ncplane* n, int y, int x, wchar_t w);
 int ncplane_putwc(struct ncplane* n, wchar_t w);
 int ncplane_putegc_yx(struct ncplane* n, int y, int x, const char* gclust, int* sbytes);
 int ncplane_putstr_aligned(struct ncplane* n, int y, ncalign_e align, const char* s);
+struct ncplane* ncplane_dup(const struct ncplane* n, void* opaque);
 void cell_init(cell* c);
 int cell_load(struct ncplane* n, cell* c, const char* gcluster);
 int cell_prime(struct ncplane* n, cell* c, const char* gcluster, uint32_t attr, uint64_t channels);
@@ -205,8 +218,8 @@ void cell_styles_on(cell* c, unsigned stylebits);
 void cell_styles_off(cell* c, unsigned stylebits);
 void cell_set_fg_default(cell* c);
 void cell_set_bg_default(cell* c);
-int cell_set_fg_alpha(cell* c, int alpha);
-int cell_set_bg_alpha(cell* c, int alpha);
+int cell_set_fg_alpha(cell* c, unsigned alpha);
+int cell_set_bg_alpha(cell* c, unsigned alpha);
 bool cell_double_wide_p(const cell* c);
 bool cell_simple_p(const cell* c);
 const char* cell_extended_gcluster(const struct ncplane* n, const cell* c);
@@ -243,26 +256,6 @@ int palette256_set_rgb(palette256* p, int idx, int r, int g, int b);
 int palette256_set(palette256* p, int idx, unsigned rgb);
 int palette256_get_rgb(const palette256* p, int idx, unsigned* r, unsigned* g, unsigned* b);
 void palette256_free(palette256* p);
-bool notcurses_canchangecolor(const struct notcurses* nc);
-struct ncdirect* ncdirect_init(const char* termtype, FILE* fp);
-int ncdirect_bg_rgb8(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
-int ncdirect_fg_rgb8(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
-int ncdirect_fg(struct ncdirect* n, unsigned rgb);
-int ncdirect_bg(struct ncdirect* n, unsigned rgb);
-int ncdirect_styles_set(struct ncdirect* n, unsigned stylebits);
-int ncdirect_styles_on(struct ncdirect* n, unsigned stylebits);
-int ncdirect_styles_off(struct ncdirect* n, unsigned stylebits);
-int ncdirect_clear(struct ncdirect* n);
-int ncdirect_stop(struct ncdirect* n);
-int ncdirect_dim_x(const struct ncdirect* nc);
-int ncdirect_dim_y(const struct ncdirect* nc);
-int ncdirect_cursor_move_yx(struct ncdirect* n, int y, int x);
-int ncdirect_cursor_enable(struct ncdirect* nc);
-int ncdirect_cursor_disable(struct ncdirect* nc);
-int ncdirect_cursor_up(struct ncdirect* nc, int num);
-int ncdirect_cursor_left(struct ncdirect* nc, int num);
-int ncdirect_cursor_right(struct ncdirect* nc, int num);
-int ncdirect_cursor_down(struct ncdirect* nc, int num);
 typedef enum {
   NCERR_SUCCESS,
   NCERR_NOMEM,
@@ -270,33 +263,49 @@ typedef enum {
   NCERR_DECODE,
   NCERR_UNIMPLEMENTED,
 } nc_err_e;
-struct ncvisual* ncplane_visual_open(struct ncplane* nc, const char* file, nc_err_e* err);
 typedef enum {
   NCSCALE_NONE,
   NCSCALE_SCALE,
   NCSCALE_STRETCH,
 } ncscale_e;
-struct ncvisual* ncvisual_open_plane(struct notcurses* nc, const char* file, nc_err_e* err, int y, int x, ncscale_e style);
-struct ncplane* ncvisual_plane(struct ncvisual* ncv);
+struct ncvisual* ncvisual_from_file(const char* file, nc_err_e* ncerr);
+struct ncvisual* ncvisual_from_rgba(const void* rgba, int rows, int rowstride, int cols);
+struct ncvisual* ncvisual_from_bgra(const void* rgba, int rows, int rowstride, int cols);
+struct ncvisual* ncvisual_from_plane(const struct ncplane* n, ncblitter_e blit, int begy, int begx, int leny, int lenx);
+int ncvisual_geom(const struct notcurses* nc, const struct ncvisual* n, const struct ncvisual_options* vopts, int* y, int* x, int* toy, int* tox);
 void ncvisual_destroy(struct ncvisual* ncv);
 nc_err_e ncvisual_decode(struct ncvisual* nc);
-int ncvisual_render(const struct ncvisual* ncv, int begy, int begx, int leny, int lenx);
+int ncvisual_rotate(struct ncvisual* n, double rads);
+int ncvisual_resize(struct ncvisual* n, int rows, int cols);
+int ncvisual_polyfill_yx(struct ncvisual* n, int y, int x, uint32_t rgba);
+struct ncplane* ncvisual_render(struct notcurses* nc, struct ncvisual* ncv, const struct ncvisual_options* vopts);
 char* ncvisual_subtitle(const struct ncvisual* ncv);
-typedef int (*streamcb)(struct notcurses* nc, struct ncvisual* ncv, void*);
-int ncvisual_stream(struct notcurses* nc, struct ncvisual* ncv, nc_err_e* err, float timescale, streamcb streamer, void* curry);
-int ncblit_bgrx(struct ncplane* nc, int placey, int placex, int linesize, const unsigned char* data, int begy, int begx, int leny, int lenx);
-int ncblit_rgba(struct ncplane* nc, int placey, int placex, int linesize, const unsigned char* data, int begy, int begx, int leny, int lenx);
-struct selector_item {
+int ncvisual_at_yx(const struct ncvisual* n, int y, int x, uint32_t* pixel);
+int ncvisual_set_yx(const struct ncvisual* n, int y, int x, uint32_t pixel);
+typedef int (*streamcb)(struct ncvisual*, struct ncvisual_options*, const struct timespec*, void*);
+int ncvisual_stream(struct notcurses* nc, struct ncvisual* ncv, nc_err_e* ncerr, float timescale, streamcb streamer, const struct ncvisual_options* vopts, void* curry);
+struct ncvisual_options {
+  struct ncplane* n;
+  ncscale_e scaling;
+  int y, x;
+  int begy, begx;
+  int leny, lenx;
+  ncblitter_e blitter;
+  uint64_t flags;
+};
+int ncblit_bgrx(const void* data, int linesize, const struct ncvisual_options *vopts);
+int ncblit_rgba(const void* data, int linesize, const struct ncvisual_options *vopts);
+struct ncselector_item {
   char* option;
   char* desc;
   size_t opcolumns;   // filled in by library
   size_t desccolumns; // filled in by library
 };
-typedef struct selector_options {
+typedef struct ncselector_options {
   char* title; // title may be NULL, inhibiting riser, saving two rows.
   char* secondary; // secondary may be NULL
   char* footer; // footer may be NULL
-  struct selector_item* items; // initial items and descriptions
+  struct ncselector_item* items; // initial items and descriptions
   unsigned itemcount; // number of initial items and descriptions
   // default item (selected at start), must be < itemcount unless 'itemcount'
   // is 0, in which case 'defidx' must also be 0
@@ -310,9 +319,9 @@ typedef struct selector_options {
   uint64_t footchannels; // secondary and footer channels
   uint64_t boxchannels;  // border channels
   uint64_t bgchannels;   // background channels, used only in body
-} selector_options;
-struct ncselector* ncselector_create(struct ncplane* n, int y, int x, const selector_options* opts);
-int ncselector_additem(struct ncselector* n, const struct selector_item* item);
+} ncselector_options;
+struct ncselector* ncselector_create(struct ncplane* n, int y, int x, const ncselector_options* opts);
+int ncselector_additem(struct ncselector* n, const struct ncselector_item* item);
 int ncselector_delitem(struct ncselector* n, const char* item);
 const char* ncselector_selected(const struct ncselector* n);
 struct ncplane* ncselector_plane(struct ncselector* n);
@@ -320,16 +329,16 @@ const char* ncselector_previtem(struct ncselector* n);
 const char* ncselector_nextitem(struct ncselector* n);
 bool ncselector_offer_input(struct ncselector* n, const struct ncinput* nc);
 void ncselector_destroy(struct ncselector* n, char** item);
-struct mselector_item {
+struct ncmselector_item {
   char* option;
   char* desc;
   bool selected;
 };
-typedef struct multiselector_options {
+typedef struct ncmultiselector_options {
   char* title; // title may be NULL, inhibiting riser, saving two rows.
   char* secondary; // secondary may be NULL
   char* footer; // footer may be NULL
-  struct mselector_item* items; // initial items, descriptions, and statuses
+  struct ncmselector_item* items; // initial items, descriptions, and statuses
   unsigned itemcount; // number of items and descriptions, can't be 0
   // maximum number of options to display at once, 0 to use all available space
   unsigned maxdisplay;
@@ -340,8 +349,8 @@ typedef struct multiselector_options {
   uint64_t footchannels; // secondary and footer channels
   uint64_t boxchannels;  // border channels
   uint64_t bgchannels;   // background channels, used only in body
-} multiselector_options;
-struct ncmultiselector* ncmultiselector_create(struct ncplane* n, int y, int x, const multiselector_options* opts);
+} ncmultiselector_options;
+struct ncmultiselector* ncmultiselector_create(struct ncplane* n, int y, int x, const ncmultiselector_options* opts);
 int ncmultiselector_selected(struct ncmultiselector* n, bool* selected, unsigned count);
 struct ncplane* ncmultiselector_plane(struct ncmultiselector* n);
 bool ncmultiselector_offer_input(struct ncmultiselector* n, const struct ncinput* nc);
@@ -357,14 +366,13 @@ struct ncmenu_section {
   ncinput shortcut;       // shortcut, will be underlined if present in name
 };
 typedef struct ncmenu_options {
-  bool bottom;              // on the bottom row, as opposed to top row
-  bool hiding;              // hide the menu when not being used
   struct ncmenu_section* sections; // array of 'sectioncount' menu_sections
   int sectioncount;         // must be positive
   uint64_t headerchannels;  // styling for header
   uint64_t sectionchannels; // styling for sections
+  unsigned flags;           // bitfield over NCMENU_OPTION_*
 } ncmenu_options;
-struct ncmenu* ncmenu_create(struct notcurses* nc, const ncmenu_options* opts);
+struct ncmenu* ncmenu_create(struct ncplane* n, const ncmenu_options* opts);
 int ncmenu_unroll(struct ncmenu* n, int sectionidx);
 int ncmenu_rollup(struct ncmenu* n);
 int ncmenu_nextsection(struct ncmenu* n);
@@ -381,14 +389,13 @@ typedef struct ncreel_options {
   int max_supported_cols;
   int max_supported_rows;
   int toff, roff, boff, loff;
-  bool infinitescroll;
-  bool circular;
   unsigned bordermask;
   uint64_t borderchan;
   unsigned tabletmask;
   uint64_t tabletchan;
   uint64_t focusedchan;
   uint64_t bgchannel;
+  unsigned flags;      // bitfield over NCREEL_OPTION_*
 } ncreel_options;
 struct ncreel* ncreel_create(struct ncplane* nc, const ncreel_options* popts, int efd);
 struct ncplane* ncreel_plane(struct ncreel* pr);
@@ -420,23 +427,12 @@ int ncplane_rotate_cw(struct ncplane* n);
 int ncplane_rotate_ccw(struct ncplane* n);
 void ncplane_translate(const struct ncplane* src, const struct ncplane* dst, int* y, int* x);
 bool ncplane_translate_abs(const struct ncplane* n, int* y, int* x);
-typedef enum {
-  NCPLOT_1x1,   // full block                █
-  NCPLOT_2x1,   // full/(upper|left) blocks  ▄█
-  NCPLOT_1x1x4, // shaded full blocks        ▓▒░█
-  NCPLOT_2x2,   // quadrants                 ▗▐ ▖▄▟▌▙█
-  NCPLOT_4x1,   // four vert/horz levels     █▆▄▂ / ▎▌▊█
-  NCPLOT_4x2,   // 4 rows, 2 cols (braille)  ⡀⡄⡆⡇⢀⣀⣄⣆⣇⢠⣠⣤⣦⣧⢰⣰⣴⣶⣷⢸⣸⣼⣾⣿
-  NCPLOT_8x1,   // eight vert/horz levels    █▇▆▅▄▃▂▁ / ▏▎▍▌▋▊▉█
-} ncgridgeom_e;
 typedef struct ncplot_options {
   uint64_t maxchannel;
   uint64_t minchannel;
-  ncgridgeom_e gridtype;
+  ncblitter_e gridtype;
   uint64_t rangex;
-  bool labelaxisd;
-  bool exponentially;
-  bool vertical_indep;
+  unsigned flags;
 } ncplot_options;
 struct ncuplot* ncuplot_create(struct ncplane* n, const ncplot_options* opts, uint64_t miny, uint64_t maxy);
 struct ncdplot* ncdplot_create(struct ncplane* n, const ncplot_options* opts, double miny, double maxy);
@@ -446,9 +442,71 @@ int ncuplot_add_sample(struct ncuplot* n, uint64_t x, uint64_t y);
 int ncdplot_add_sample(struct ncdplot* n, uint64_t x, double y);
 int ncuplot_set_sample(struct ncuplot* n, uint64_t x, uint64_t y);
 int ncdplot_set_sample(struct ncdplot* n, uint64_t x, double y);
+int ncuplot_sample(const struct ncuplot* n, uint64_t x, uint64_t* y);
+int ncdplot_sample(const struct ncdplot* n, uint64_t x, double* y);
 void ncuplot_destroy(struct ncuplot* n);
 void ncdplot_destroy(struct ncdplot* n);
 bool ncplane_set_scrolling(struct ncplane* n, bool scrollp);
+typedef struct ncfdplane_options {
+  void* curry; // parameter provided to callbacks
+  bool follow; // keep reading after hitting end? (think tail -f)
+} ncfdplane_options;
+typedef int(ncfdplane_callback)(struct ncfdplane* n, const void* buf, size_t s, void* curry);
+typedef int(ncfdplane_done_cb)(struct ncfdplane* n, int fderrno, void* curry);
+struct ncfdplane* ncfdplane_create(struct ncplane* n, const ncfdplane_options* opts, int fd, ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn);
+struct ncplane* ncfdplane_plane(struct ncfdplane* n);
+int ncfdplane_destroy(struct ncfdplane* n);
+typedef struct ncsubproc_options {
+  void* curry;
+  uint64_t restart_period;  // restart this many seconds after an exit (watch)
+} ncsubproc_options;
+struct ncsubproc* ncsubproc_createv(struct ncplane* n, const ncsubproc_options* opts, const char* bin, char* const arg[], ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn);
+struct ncsubproc* ncsubproc_createvp(struct ncplane* n, const ncsubproc_options* opts, const char* bin, char* const arg[], ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn);
+struct ncsubproc* ncsubproc_createvpe(struct ncplane* n, const ncsubproc_options* opts, const char* bin, char* const arg[], char* const env[], ncfdplane_callback cbfxn, ncfdplane_done_cb donecbfxn);
+struct ncplane* ncsubproc_plane(struct ncsubproc* n);
+int ncsubproc_destroy(struct ncsubproc* n);
+typedef struct ncreader_options {
+  uint64_t tchannels; // channels used for input
+  uint64_t echannels; // channels used for empty space
+  uint32_t tattrword; // attributes used for input
+  uint32_t eattrword; // attributes used for empty space
+  char* egc;          // egc used for empty space
+  int physrows;
+  int physcols;
+  unsigned flags;     // bitfield over NCREADER_OPTION_*
+} ncreader_options;
+struct ncreader* ncreader_create(struct ncplane* n, int y, int x, const ncreader_options* opts);
+int ncreader_clear(struct ncreader* n);
+struct ncplane* ncreader_plane(struct ncreader* n);
+bool ncreader_offer_input(struct ncreader* n, const struct ncinput* ni);
+char* ncreader_contents(const struct ncreader* n);
+void ncreader_destroy(struct ncreader* n, char** contents);
+int ncplane_puttext(struct ncplane* n, int y, ncalign_e align, const char* text, size_t* bytes);
+int ncplane_putnstr_yx(struct ncplane* n, int y, int x, size_t s, const char* gclusters);
+int ncplane_putnstr_aligned(struct ncplane* n, int y, ncalign_e align, size_t s, const char* gclustarr);
+int ncplane_qrcode(struct ncplane* n, ncblitter_e blitter, int* ymax, int* xmax, const void* data, size_t len);
+struct ncdirect* ncdirect_init(const char* termtype, FILE* fp);
+int ncdirect_bg_rgb(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
+int ncdirect_fg_rgb(struct ncdirect* n, unsigned r, unsigned g, unsigned b);
+int ncdirect_palette_size(const struct ncdirect* nc);
+int ncdirect_putc(struct ncdirect* nc, uint64_t channels, const char* egc);
+int ncdirect_fg(struct ncdirect* n, unsigned rgb);
+int ncdirect_bg(struct ncdirect* n, unsigned rgb);
+int ncdirect_styles_set(struct ncdirect* n, unsigned stylebits);
+int ncdirect_styles_on(struct ncdirect* n, unsigned stylebits);
+int ncdirect_styles_off(struct ncdirect* n, unsigned stylebits);
+int ncdirect_clear(struct ncdirect* n);
+int ncdirect_stop(struct ncdirect* n);
+int ncdirect_dim_x(const struct ncdirect* nc);
+int ncdirect_dim_y(const struct ncdirect* nc);
+int ncdirect_cursor_move_yx(struct ncdirect* n, int y, int x);
+int ncdirect_cursor_enable(struct ncdirect* nc);
+int ncdirect_cursor_disable(struct ncdirect* nc);
+int ncdirect_cursor_up(struct ncdirect* nc, int num);
+int ncdirect_cursor_left(struct ncdirect* nc, int num);
+int ncdirect_cursor_right(struct ncdirect* nc, int num);
+int ncdirect_cursor_down(struct ncdirect* nc, int num);
+nc_err_e ncdirect_render_image(struct ncdirect* n, const char* filename, ncalign_e align, ncblitter_e blitter, ncscale_e scale);
 """)
 
 if __name__ == "__main__":
